<?php
/**
 * @package   Yandex id
 * @author    Outcode https://outcode.ru/
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Uri\Uri;

class YandexidControllerWrapper extends JControllerLegacy{

  function __construct($config = array()){
    JLoader::register('YandexApi', JPATH_ROOT . '/components/com_yandexid/YandexApi.php');    
    $this->YandexApi = new YandexApi();
    $this->uri = Uri::getInstance();

    parent::__construct( $config );
  }

  function display($cachable = false, $urlparams = false){  

    $app = JFactory::getApplication();
    $content = trim(file_get_contents('php://input'));
    $api = $app->input->get('api', '', 'string');
    $access_token = $this->input->get('access_token', '', 'STRING');

    switch ($api) {
      case 'create_oauth_app':        
        $this->getOauthToken();
        $this->createOauthApp($content);
        break;
        case 'oauth':        
        $this->oauthPage();
        break;
      case 'authorization_user':        
        $this->authorizationUser($access_token);
        break;
      default;
        JError::raiseError( 404, JText::_('COM_YANDEXPAY_PAGE_NOT_FOUND') );
        break;
    }  
    
    jexit();    
  }
  
  function getOauthToken(){
    $params = JComponentHelper::getParams('com_yandexid');
    
    $oauth_token = $params->get('yandexid_oauth_token', '');
    $expires_in = $params->get('yandexid_expires_oauth_token', '');

    $expires_in = ($expires_in == false) ? 0 : $expires_in;
    
    if($oauth_token == false || $expires_in <= time()){
      $result = $this->YandexApi->getToken();
      
      if($result != false){
        $params->set( 'yandexid_oauth_token', $result->access_token );
        $params->set( 'yandexid_expires_oauth_token', time() + $result->expires_in );
        
        $componentid = JComponentHelper::getComponent('com_yandexid')->id;
        $table = JTable::getInstance('extension');
        $table->load($componentid);
        $table->bind(array('params' => $params->toString()));
        $table->check();
        $table->store();
      }
    }
  }
  
  function createOauthApp($content){
    $params = JComponentHelper::getParams('com_yandexid');
    
    $yandexid_title = $params->get('title', '');
    $oauth_token = $params->get('yandexid_oauth_token', '');
    $expires_in = $params->get('yandexid_expires_oauth_token', '');
    $client_id = $params->get('yandexid_app_client_id', '');

    if($yandexid_title == false){
      echo json_encode([
        'error' => true,
        'message' => JText::_("COM_YANDEXID_TXT_100")
        ]);
      jexit(); 
    }

    if($oauth_token == false || $expires_in == false || $expires_in <= time()){
      echo json_encode([
        'error' => true,
        'message' => JText::_("COM_YANDEXID_TXT_101")
        ]);
      jexit(); 
    }  

    $data = [
      'title' => $yandexid_title,
      'scopes' => $this->get_scopes($params),
      'platforms' => 'web',
      'redirect_uris' =>  $this->uri->base().'index.php/component/com_yandexid/yandexid/oauth'
    ];

    if($client_id == false){  // Создаем приложение    
      $result = $this->YandexApi->createOauthApp($oauth_token, $data);
      
      if($result != false){
        $params->set( 'yandexid_app_client_id', $result->client_id );
        $params->set( 'yandexid_app_client_secret', $result->client_secret );
        
        $componentid = JComponentHelper::getComponent('com_yandexid')->id;
        $table = JTable::getInstance('extension');
        $table->load($componentid);
        $table->bind(array('params' => $params->toString()));
        $table->check();
        $table->store();
        
        echo json_encode([
          'error' => false,
          'message' => JText::_("COM_YANDEXID_TXT_102"),
          ]);
        jexit(); 
      }
    }else{ // Редактируем приложение
      $data['client_id'] = $client_id;
      $result = $this->YandexApi->editOauthApp($oauth_token, $data);
      
      if($result != false){
        echo json_encode([
          'error' => false,
          'message' => JText::_("COM_YANDEXID_TXT_103"),
          ]);
        jexit(); 
      }
    }
    
    echo json_encode([
      'error' => true,
      'message' => JText::_("COM_YANDEXID_TXT_104")
      ]);
    jexit(); 
  }
  
  function oauthPage(){
	  $params = [
      'originUrl' => $this->uri->base().'index.php/component/com_yandexid/yandexid/oauth'
    ];

    extract($params);
    include JPATH_COMPONENT . '/views/oauth.php';
    $buffer = ob_get_contents();
    ob_end_clean();
    echo $buffer;      
    jexit(); 
  }
  
  function authorizationUser($access_token = ''){

    if($access_token != ''){
      $user_data = $this->YandexApi->getUserInfo($access_token);  
      
      if(isset($user_data->default_email)){
        $email = $user_data->default_email;
      }else{
        $email = null;
      }  
    }
    
    if(is_null($email)){
      echo json_encode([
      'error' => true,
        'message' => 'Не возможно авторизовать пользователя. Войдите в аккаунт с email'
        ]);
        
      die();  
    } 

    $user_id = $this->getUserIdByEmail($email);
    
    if(is_null($user_id)){
      $user_id = $this->createUser($user_data);
    }
    
    if(is_numeric($user_id)){
    
      $app = JFactory::getApplication();
      $user = JFactory::getUser($user_id);
      $session = JFactory::getSession();
      
      $session->set('user', $user);
      $credentials = ['username' => $user->username];
      $app->login($credentials, []);
      echo json_encode([
        'error' => false
        ]);
        
      die();
    }

  }  
  
  public function createUser($user_data){
    $user = new JUser;
    $pass = JUserHelper::genRandomPassword();
    $params = JComponentHelper::getParams('com_yandexid');

    $email = $user_data->default_email;

    $userData = [
      'name' => $email,
      'username' => $email,
      'password' => $pass,
      'password2' => $pass,
      'email' => $email,
      'familiya' => 'test',
      'groups' => [ $params->get('group', 1) ]
    ];
    
    $user->bind( $userData );    
    $user->save();

    if ( $user->save() ) {

      $use_login_info = $params->get('scope_login_info', 0);
      $use_phone = $params->get('scope_login_default_phone', 0);
      $use_birthday = $params->get('scope_login_birthday', 0);      
      
      if($use_login_info == 1){
        $first_name = $params->get('first_name', 'no');
        $last_name = $params->get('last_name', 'no');
        
        if($first_name != 'no'){
          $value = isset($user_data->first_name) ? $user_data->first_name : '';
          $this->saveCustomField($first_name, $user->id, $value);
        }
        
        if($last_name != 'no'){
          $value = isset($user_data->last_name) ? $user_data->last_name : '';
          $this->saveCustomField($last_name, $user->id, $value);
        }
      }
      
      if($use_phone == 1){
        $default_phone = $params->get('default_phone', 'no');

        if($default_phone != 'no'){
          $value = isset($user_data->default_phone->number) ? $user_data->default_phone->number : '';
          $this->saveCustomField($default_phone, $user->id, $value);
        }
      }
      
      if($use_birthday == 1){
        $login_birthday = $params->get('login_birthday', 'no');

        if($login_birthday != 'no'){
          $value = isset($user_data->birthday) ? $user_data->birthday : '';
          $this->saveCustomField($login_birthday, $user->id, $value);
        }
      }
    
      return $user->id;
    } 
    
    return false;
  }
  
  public function saveCustomField($field_id, $user_id, $value){

		$db = JFactory::getDbo();
    $query = $db->getQuery(true);    

    $query->delete($query->qn('#__fields_values'))
      ->where($query->qn('field_id') . ' = ' . (int) $field_id)
      ->where($query->qn('item_id') . ' = ' . $query->q($user_id));

		$db->setQuery($query)->execute();
    
    $newObj = new stdClass;

    $newObj->field_id = (int) $field_id;
    $newObj->item_id  = $user_id;
    $newObj->value    = $value;

    $db->insertObject('#__fields_values', $newObj);    
  }
  
  public function getUserIdByEmail($email){
    $db = JFactory::getDbo();
    $query = $db->getQuery(true);
    $query = 'SELECT id FROM #__users WHERE email = ' . $db->Quote($email);
    $db->setQuery($query, 0, 1);

    return $db->loadResult();   
  }    
  
  public function get_scopes($params){
    $options = [
      'scope_login_info' => 'login:info',
      'scope_login_email' => 'login:email',
      'scope_login_default_phone' => 'login:default_phone',
      'scope_login_birthday' => 'login:birthday'
    ];  
    
    $result = [];
    
    foreach($options as $option => $scope){
      $value = $params->get($option, '');
      
      if($value == 1 || $scope == 'login:email'){
        $result[] = $scope;
      }
    }

    return implode('&scopes=', $result);
  }   
}